/** BEGIN COPYRIGHT BLOCK
 * This Program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; version 2 of the License.
 *
 * This Program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this Program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#include "coolkey-api.h"
#include "rhCoolKey.h"
#include <string>


static rhCoolKey *coolkey = NULL;
static const char * coolkeyDbusName = NULL;
static const char * configFilePathName = NULL;
static CoolKeyListener *listener = NULL;

#define DEF_TOKEN_TYPE "userKey"


int notify(CoolKeyListener *listener, unsigned long keyType, const char *keyID,
		unsigned long keyState, unsigned long data, const char *strData) {
	CoolKeyDispatch clientListener = listener->GetClientListener();

	/* notify our instance of rhCoolKey */

	coolkey->RhNotifyKeyStateChange(keyType, keyID, keyState, data, strData);

	if (clientListener) {
		(clientListener)((CoolKeyListener *) listener->GetClient(), keyType,
				keyID, keyState, data, strData);
	}
	return S_OK;
}

void coolkey_destroy() {
	if (coolkey != NULL) {
                if(listener) {
                    coolkey->UnregisterCoolKeyListener(listener);
                }
		coolkey->ShutDownInstance();
		delete coolkey;
		coolkey = NULL;
	}
}

void coolkey_init(const char *db_dir, const char *configFileName, CoolKeyNotifyCallback cb,
		void* notifyClient, const char* dbusUniqueName) {

	coolkeyDbusName = dbusUniqueName;
	if (coolkey == NULL) {
		coolkey = new rhCoolKey(db_dir,configFileName);

		listener = new CoolKeyListener();
		listener->SetListener((CoolKeyDispatch) cb, (CoolKeyDispatch) notify);
		listener->SetClient(notifyClient);
		coolkey->RegisterCoolKeyListener(listener);
	}
}

char *coolkey_get_phone_home(char *url) {
   if(!url) {
       return NULL;
   }
   if (coolkey != NULL) {
       char *_retval = NULL;
       coolkey->GetCoolKeyPhoneHomeData(url, &_retval);
       return _retval;
   } else {
      return NULL;
   } 
}

/* get a string with all the certs detail for a token */

char *coolkey_get_certs_info(int keyType, const char *keyID) {

    string str_result;
    if (coolkey == NULL) {
	    return NULL;
    }

    char *result = NULL;
    char **names = NULL;
    PRUint32 count = 0;
    HRESULT res =  coolkey->GetCoolKeyCertNicknames(keyType, keyID, &count, &names);

    if(count > 0 && res == S_OK) {
        for(int i = 0 ; i < count ; i++) {
	    char *curName = names[i];

	    if(curName) {
	        char *certDetail = NULL;
		str_result = str_result + curName + "\n" ;
	        res = coolkey->GetCoolKeyCertInfo(keyType, keyID, curName, &certDetail);
	        if(res == S_OK && certDetail != NULL) {
		    str_result = str_result + certDetail + "\n";
		    PL_strfree(certDetail);
		    certDetail = NULL;
	        }
	    }
	}
    }

    if(str_result.c_str()) {
        result = PL_strdup((char *) str_result.c_str());
    }

    for(int i = 0 ; i < count ; i++) {
        if(names[i]) {
	    PL_strfree(names[i]);
	    names[i] = NULL;
	}
    }

    PR_Free(names);
    names = NULL;

    return result;
}


/* get a block of data about a token in a structure format */
tokenInfo *coolkey_get_token_info(int keyType, const char *keyID) {

	if (coolkey == NULL) {
		return NULL;
	}
	tokenInfo *tInfo = (tokenInfo *) malloc(sizeof(tokenInfo));

	if (tInfo == NULL) {
		exit(1);
	}

	tInfo->atr = NULL;
	tInfo->issuerInfo = NULL;
	tInfo->issuer = NULL;
	tInfo->issuedTo = NULL;
	tInfo->status = 0;
        tInfo->isACoolKey = 0;

	coolkey->GetCoolKeyATR(keyType, keyID, &tInfo->atr);

	coolkey->GetCoolKeyIssuerInfo(keyType, keyID, &tInfo->issuerInfo);

	coolkey->GetCoolKeyIssuer(keyType, keyID, &tInfo->issuer);

	coolkey->GetCoolKeyIssuedTo(keyType, keyID, &tInfo->issuedTo);

	coolkey->GetCoolKeyStatus(keyType, keyID, &tInfo->status);

        bool isACoolKey = false;
        coolkey->GetCoolKeyIsReallyCoolKey(keyType, keyID, &isACoolKey);
        tInfo->isACoolKey = (int) isACoolKey;

	return tInfo;
}

int coolkey_cancel_token_operation(int keyType, const char *keyId) {

    if(coolkey == NULL) {
        return E_FAIL;
    }

    int ret = E_FAIL;
    ret = coolkey->CancelCoolKeyOperation(keyType, keyId);

    return S_OK;
}

int coolkey_format_token(int keyType, const char *keyId, const char *screenName, const char *pin, const char *screenNamePin, const char *tokenCode) {

    if (coolkey == NULL) {
        return E_FAIL;
    }

    int ret = E_FAIL;

    ret = coolkey->FormatCoolKey(keyType, keyId, DEF_TOKEN_TYPE, screenName, pin, screenNamePin, tokenCode);

    return S_OK;

}

int coolkey_enroll_token(int keyType, const char *keyId, const char *screenName, const char *pin, const char *screenNamePin, const char *tokenCode ) {

    if (coolkey == NULL) {
        return E_FAIL;
    }

    int ret = E_FAIL;

    ret = coolkey->EnrollCoolKey(keyType, keyId, DEF_TOKEN_TYPE, screenName, pin, screenNamePin, tokenCode);


    return S_OK;
}

int coolkey_reset_token_pin(int keyType, const char *keyId, const char *screenName, const char *pin, const char *screenNamePin, const char *tokenCode) {


    if (coolkey == NULL) {
        return E_FAIL;
    }

    int ret = E_FAIL;

    ret = coolkey->ResetCoolKeyPIN(keyType, keyId,  screenName, pin, screenNamePin);


    return S_OK;
}

void coolkey_free_token_info(tokenInfo *tInfo) {
	if (tInfo) {
		if (tInfo->atr) {
			PL_strfree(tInfo->atr);
			tInfo->atr = NULL;
		}

		if (tInfo->issuerInfo) {
			PL_strfree(tInfo->issuerInfo);
			tInfo->issuerInfo = NULL;
		}

		if (tInfo->issuer) {
			PL_strfree(tInfo->issuer);
			tInfo->issuer = NULL;
		}

		if (tInfo->issuedTo) {
			PL_strfree(tInfo->issuedTo);
			tInfo->issuedTo = NULL;
		}

		free(tInfo);
	}
}
